@@ -22,4 +22,5 @@ class ESPEIValidator(Validator):
 with open(os.path.join(MODULE_DIR, 'input-schema.yaml')) as f:
     schema = ESPEIValidator(yaml.load(f))
 
-from espei.paramselect import fit
+from espei.paramselect import generate_parameters, mcmc_fit
+from espei.espei_script import run_espei
